@@ -45,10 +45,10 @@ namespace draft4 {
         std::unordered_map<std::string,keyword_factory_type> keyword_factory_map_;
 
     public:
-        schema_builder_4(const schema_builder_factory_type& builder_factory, 
+        schema_builder_4(Json&& sch, const schema_builder_factory_type& builder_factory, 
             evaluation_options options, schema_store_type* schema_store_ptr,
             const std::vector<schema_resolver<Json>>& resolvers) 
-            : schema_builder<Json>(schema_version::draft4(), builder_factory, options, schema_store_ptr, resolvers)
+            : schema_builder<Json>(schema_version::draft4(), std::move(sch), builder_factory, options, schema_store_ptr, resolvers)
         {
             init();
         }
@@ -61,54 +61,54 @@ namespace draft4 {
         void init()
         {
             keyword_factory_map_.emplace("type", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_type_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_type_validator(context, sch, parent);});
             keyword_factory_map_.emplace("contentEncoding", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_content_encoding_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_content_encoding_validator(context, sch, parent);});
             keyword_factory_map_.emplace("contentMediaType", 
                 [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_content_media_type_validator(context, sch, parent);});
             if (this->options().require_format_validation())
             {
                 keyword_factory_map_.emplace("format", 
-                    [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_format_validator(context, sch);});
+                    [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_format_validator(context, sch, parent);});
             }
 #if defined(JSONCONS_HAS_STD_REGEX)
             keyword_factory_map_.emplace("pattern", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_pattern_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_pattern_validator(context, sch, parent);});
 #endif
             keyword_factory_map_.emplace("maxItems", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_max_items_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_max_items_validator(context, sch, parent);});
             keyword_factory_map_.emplace("minItems", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_min_items_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_min_items_validator(context, sch, parent);});
             keyword_factory_map_.emplace("maxProperties", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_max_properties_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_max_properties_validator(context, sch, parent);});
             keyword_factory_map_.emplace("minProperties", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_min_properties_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_min_properties_validator(context, sch, parent);});
             keyword_factory_map_.emplace("uniqueItems", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_unique_items_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_unique_items_validator(context, sch, parent);});
             keyword_factory_map_.emplace("maxLength", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_max_length_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_max_length_validator(context, sch, parent);});
             keyword_factory_map_.emplace("minLength", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_min_length_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_min_length_validator(context, sch, parent);});
             keyword_factory_map_.emplace("not", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type& anchor_dict){return this->make_not_validator(context, sch, anchor_dict);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type& anchor_dict){return this->make_not_validator(context, sch, parent, anchor_dict);});
             keyword_factory_map_.emplace("maximum", 
                 [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return make_maximum_validator_4(context, sch, parent);});
             keyword_factory_map_.emplace("minimum", 
                 [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return make_minimum_validator_4(context, sch, parent);});
             keyword_factory_map_.emplace("multipleOf", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_multiple_of_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_multiple_of_validator(context, sch, parent);});
             keyword_factory_map_.emplace("enum", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_enum_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_enum_validator(context, sch, parent);});
             keyword_factory_map_.emplace("allOf", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type& anchor_dict){return this->make_all_of_validator(context, sch, anchor_dict);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type& anchor_dict){return this->make_all_of_validator(context, sch, parent, anchor_dict);});
             keyword_factory_map_.emplace("anyOf", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type& anchor_dict){return this->make_any_of_validator(context, sch, anchor_dict);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type& anchor_dict){return this->make_any_of_validator(context, sch, parent, anchor_dict);});
             keyword_factory_map_.emplace("oneOf", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type& anchor_dict){return this->make_one_of_validator(context, sch, anchor_dict);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type& anchor_dict){return this->make_one_of_validator(context, sch, parent, anchor_dict);});
             keyword_factory_map_.emplace("dependencies", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type& anchor_dict){return this->make_dependencies_validator(context, sch, anchor_dict);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type& anchor_dict){return this->make_dependencies_validator(context, sch, parent, anchor_dict);});
             keyword_factory_map_.emplace("required", 
-                [&](const compilation_context& context, const Json& sch, const Json&, anchor_uri_map_type&){return this->make_required_validator(context, sch);});
+                [&](const compilation_context& context, const Json& sch, const Json& parent, anchor_uri_map_type&){return this->make_required_validator(context, sch, parent);});
         }
 
         schema_validator_type make_schema_validator( 
@@ -153,7 +153,7 @@ namespace draft4 {
                         Json default_value{ jsoncons::null_type() };
                         uri_wrapper relative(it->value().template as<std::string>()); 
                         auto id = relative.resolve(uri_wrapper{ context.get_base_uri() });
-                        validators.push_back(this->get_or_create_reference(id));
+                        validators.push_back(this->get_or_create_reference(sch, id));
                         schema_validator_ptr = jsoncons::make_unique<object_schema_validator<Json>>(
                             new_context.get_base_uri(), context.id(),
                             std::move(validators), std::move(defs), std::move(default_value));
@@ -225,7 +225,7 @@ namespace draft4 {
             it = sch.find("properties");
             if (it != sch.object_range().end()) 
             {
-                properties = this->make_properties_validator(context, it->value(), anchor_dict);
+                properties = this->make_properties_validator(context, it->value(), sch, anchor_dict);
             }
             std::unique_ptr<pattern_properties_validator<Json>> pattern_properties;
 
@@ -233,14 +233,14 @@ namespace draft4 {
             it = sch.find("patternProperties");
             if (it != sch.object_range().end())
             {
-                pattern_properties = make_pattern_properties_validator(context, it->value(), anchor_dict);
+                pattern_properties = make_pattern_properties_validator(context, it->value(), sch, anchor_dict);
             }
     #endif
 
             it = sch.find("additionalProperties");
             if (it != sch.object_range().end()) 
             {
-                validators.emplace_back(this->make_additional_properties_validator(context, it->value(), 
+                validators.emplace_back(this->make_additional_properties_validator(context, it->value(), sch,
                     std::move(properties), std::move(pattern_properties), anchor_dict));
             }
             else
@@ -263,22 +263,22 @@ namespace draft4 {
 
                 if (it->value().type() == json_type::array_value) 
                 {
-                    validators.emplace_back(this->make_prefix_items_validator_07(context, sch, it->value(), anchor_dict));
+                    validators.emplace_back(this->make_prefix_items_validator_07(context, it->value(), sch, anchor_dict));
                 } 
                 else if (it->value().type() == json_type::object_value ||
                            it->value().type() == json_type::bool_value)
                 {
-                    validators.emplace_back(this->make_items_validator("items", context, it->value(), anchor_dict));
+                    validators.emplace_back(this->make_items_validator("items", context, it->value(), sch, anchor_dict));
                 }
             }
-            return jsoncons::make_unique<object_schema_validator<Json>>(context.get_base_uri(), std::move(id),
-                std::move(validators), std::move(defs), std::move(default_value));
+            return jsoncons::make_unique<object_schema_validator<Json>>(context.get_base_uri(), 
+                std::move(id), std::move(validators), std::move(defs), std::move(default_value));
         }
 
 #if defined(JSONCONS_HAS_STD_REGEX)
                 
         std::unique_ptr<pattern_properties_validator<Json>> make_pattern_properties_validator(const compilation_context& context, 
-            const Json& sch, anchor_uri_map_type& anchor_dict)
+            const Json& sch, const Json& parent, anchor_uri_map_type& anchor_dict)
         {
             uri schema_location = context.get_base_uri();
             std::vector<std::pair<std::regex, schema_validator_type>> pattern_properties;
@@ -292,7 +292,7 @@ namespace draft4 {
                         make_schema_validator(context, prop.value(), sub_keys, anchor_dict)));
             }
 
-            return jsoncons::make_unique<pattern_properties_validator<Json>>( std::move(schema_location),
+            return jsoncons::make_unique<pattern_properties_validator<Json>>(parent, std::move(schema_location),
                 std::move(pattern_properties));
         }
 #endif
@@ -319,11 +319,11 @@ namespace draft4 {
             }
             if (is_exclusive)
             {
-                return jsoncons::make_unique<exclusive_maximum_validator<Json>>(schema_location, sch);
+                return jsoncons::make_unique<exclusive_maximum_validator<Json>>(parent, schema_location, sch);
             }
             else
             {
-                return jsoncons::make_unique<maximum_validator<Json>>(schema_location, sch);
+                return jsoncons::make_unique<maximum_validator<Json>>(parent, schema_location, sch);
             }
         }
 
@@ -349,11 +349,11 @@ namespace draft4 {
             }
             if (is_exclusive)
             {
-                return jsoncons::make_unique<exclusive_minimum_validator<Json>>( schema_location, sch);
+                return jsoncons::make_unique<exclusive_minimum_validator<Json>>(parent, schema_location, sch);
             }
             else
             {
-                return jsoncons::make_unique<minimum_validator<Json>>( schema_location, sch);
+                return jsoncons::make_unique<minimum_validator<Json>>(parent, schema_location, sch);
             }
         }
 
