@@ -2,10 +2,8 @@
 // Distributed under Boost license
 
 #include <catch/catch.hpp>
-#include <common/FreeListAllocator.hpp>
 #include <jsoncons/json.hpp>
 #include <map>
-#include <scoped_allocator>
 #include <vector>
 
 namespace 
@@ -159,7 +157,7 @@ TEST_CASE("convert_map_test")
 
 TEST_CASE("convert_array_test")
 {
-    std::array<double,4> v{1,2,3,4};
+    std::array<double,4> v{{1,2,3,4}};
 
     std::string s;
     jsoncons::encode_json(v,s);
@@ -276,6 +274,9 @@ TEST_CASE("encode/decode map with integer key")
 
 #if defined(JSONCONS_HAS_STATEFUL_ALLOCATOR) && JSONCONS_HAS_STATEFUL_ALLOCATOR == 1
 
+#include <scoped_allocator>
+#include <common/FreeListAllocator.hpp>
+
 template<typename T>
 using MyScopedAllocator = std::scoped_allocator_adaptor<FreeListAllocator<T>>;
 
